#!/bin/bash
#--------------------------------------------------------------------------
# Creation: 04.04.2021 JED
# Last Update: 15.04.2023 OJae
#
# Copyright (c) 2022-2023 Olaf Jaehrling eisfair(at)ojaehrling(dot)de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License
#--------------------------------------------------------------------------
#
# Functions:
#   init [force]                       - create new database, optionally
#                                        force overwrite
#   add ip-address [date: YYYY-MM-DD]  - add entry to database
#                                        Default: current date
#               e.g. '193.126.6.2' '2021-04-01'
#               e.g. '2001:0:7d38:5ab8:1c48:3a1c:a95a:b1c1' '2021-03-31'
#   cleanse [keep-number-of-days]      - delete all entries older than
#                                        the given number of days.
#                                        Default: 30 days
#   compare                            - compare content of two tables
#   print                              - output all ip addresses in
#                                        sorted order
#   show                               - show if ip address in database
#

if [ -f /etc/config.d/brute_force_blocking ]
then
    . /etc/config.d/brute_force_blocking
fi
if [ -f /etc/default/brute_force_blocking ]
then
    . /etc/default/brute_force_blocking
fi
if [ -f /usr/local/brute_force_blocking/configfile ]
then
    . /usr/local/brute_force_blocking/configfile
fi

work_path_init=/usr/local/brute_force_blocking
work_path=${bfbhome}
sqlite_bin=`which sqlite3`
DB_NAME=${work_path}/bfb.db
DB_NAME_INIT=${work_path_init}/bfb.db
DB_TABLE=bfb_blocks
DB_TABLE_Z=bfb_blocks_zahl
DB_TABLE_TMP=bfb_blocks_tmp
tmp_file=/tmp/bfb-cmd.$$
threshold_days=$cleanseday
max_lock_wait=30

COUNT=1;while ls /$bfbhome/bfb-db.lck >/dev/null 2>&1; do echo $COUNT sec;sleep 1; COUNT=`expr $COUNT + 1`; done
#while ls /$bfbhome/bfb-db.lck >/dev/null 2>&1; do sleep 1; done

if [ -d $bfbhome ]
then
    touch $bfbhome/bfb-db.lck
else
    exit 0
fi

case $1 in 
    init )
        # create new database
        if [ -f ${DB_NAME_INIT} -a "$2" = 'force' ]
        then
            # force database overwrite
            rm -f ${DB_NAME_INIT}
        fi

        if [ -f ${DB_NAME_INIT} ]
        then
            echo "ERROR: database '${DB_NAME_INIT}' already exists!"
        else
            rm -f ${tmp_file}

            # create database table structure
            (
                echo "CREATE TABLE IF NOT EXISTS ${DB_TABLE} ("
                echo "  'ipaddr'   VARCHAR(45) PRIMARY KEY,"
                echo "  'count'    VARCHAR(8),"
                echo "  'reason'   TEXT,"
                echo "  'lastused' TEXT"
                echo ");"
                echo "CREATE TABLE IF NOT EXISTS ${DB_TABLE_Z} ("
                echo "  'ipaddr'   VARCHAR(45) PRIMARY KEY,"
                echo "  'count'    VARCHAR(8),"
                echo "  'reason'   TEXT,"
                echo "  'lastused' TEXT"
                echo ");"
            ) > ${tmp_file}

            # execute sql command
            ${sqlite_bin} ${DB_NAME_INIT} < ${tmp_file}

            rm -f ${tmp_file}

            # DEBUG - output table content
            #${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE};"
            if [ "$work_path_init" != "$work_path" ]
            then
                if [ -d "$work_path" ]
                then
                    cp ${DB_NAME_INIT} ${DB_NAME}
                fi
            fi

        fi
        rm -f /$bfbhome/bfb-db.lck

        ;;
    add )
        # add/update row - anzahl und lastdate gleich und unten bei install getrennt behandelt zu werden
        ipaddr="$2"
        reason="$3"
        lastdate="$4"
        anzahl=$4
        lastdate2=$5
        
        if [ "$reason" = "" ]
        then
            reason=haendisch
        elif [ "$reason" = "install" ]
        then
                work_path=/usr/local/brute_force_blocking
                DB_NAME=${work_path}/bfb.db
                if [ "$lastdate2" = "" ]
                then
                    lastdate="`date +'%Y-%m-%d'`"
                else
                    lastdate=$lastdate2
                fi
        fi
        

        if [ -z "${ipaddr}" ]
        then
            echo "ERROR: ip address missing"
            exit 1
        fi

        if [ -z "${lastdate}" ]
        then
            # use current date
            lastdate="`date +'%Y-%m-%d'`"
        fi

        if [ ! -f ${DB_NAME} ]
        then
            echo "ERROR: database '${DB_NAME}' doesn't exist!"
        else
            rm -f ${tmp_file}

            if [ "$reason" = "install" ]
            then
                # da die ip vorher geblockt war wird sie als solche auch wieder in die zaehlertabelle eingetragen
                reason=block
                (
                    echo "INSERT INTO ${DB_TABLE_Z} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', '${reason}',${anzahl})"
                    echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
                ) > ${tmp_file}
            else
                # insert/update database entry
                (
                    echo "INSERT INTO ${DB_TABLE} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', '${reason}',1)"
                    echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
                    echo "INSERT INTO ${DB_TABLE_Z} (ipaddr, lastused, reason, count) VALUES ('${ipaddr}', '${lastdate}', '${reason}',1)"
                    echo "  ON CONFLICT (ipaddr) DO UPDATE SET lastused = '${lastdate}', count = count + 1 WHERE ipaddr = '${ipaddr}';"
                ) > ${tmp_file}
            fi

            # execute sql command
            ${sqlite_bin} ${DB_NAME} < ${tmp_file}
            rm -f ${tmp_file}
        fi
        rm -f /$bfbhome/bfb-db.lck
        ;;
    cleanse )
        # DELETE FROM scans WHERE scanned < DATETIME('now', '-30 day')
        if [ ! -f ${DB_NAME} ]
        then
            echo "ERROR: database '${DB_NAME}' doesn't exist!"
        else
            rm -f ${tmp_file}

            # execute sql command
            ${sqlite_bin} ${DB_NAME} "DELETE FROM ${DB_TABLE_Z} WHERE lastused < DATE('NOW', '-${threshold_days} days');"
        fi
        rm -f /$bfbhome/bfb-db.lck
        ;;

    del|delete )
            ip=$2
            ${sqlite_bin} ${DB_NAME} "DELETE FROM ${DB_TABLE} WHERE ipaddr = '${ip}';"

            rm -f /$bfbhome/bfb-db.lck
        ;;

    delproactive|dellong )
            ip=$2
            initd=$3
            ${sqlite_bin} ${DB_NAME} "DELETE FROM ${DB_TABLE_Z} WHERE ipaddr = '${ip}';"

            rm -f /$bfbhome/bfb-db.lck
            if [ "$initd" != "initd" ]
            then
                /usr/local/brute_force_blocking/brute_force_blocking_unblock_single.sh "${ip}" dellong 2>/dev/null
            fi
            rm -f /$bfbhome/bfb-db.lck
        ;;

    delfree )
            ${sqlite_bin} ${DB_NAME} "DELETE FROM ${DB_TABLE} WHERE reason = 'free';"

            rm -f /$bfbhome/bfb-db.lck
        ;;

    delblock )
             ${sqlite_bin} ${DB_NAME} "DELETE FROM ${DB_TABLE} WHERE reason = 'block';"

             rm -f /$bfbhome/bfb-db.lck
        ;;
    delslow )
             ${sqlite_bin} ${DB_NAME} "DELETE FROM ${DB_TABLE} WHERE reason = 'slow';"

             rm -f /$bfbhome/bfb-db.lck
        ;;

    compare )
        echo "not implementet yet"
        rm -f /$bfbhome/bfb-db.lck
        exit 0
        if [ ! -f ${DB_NAME} ]
        then
            echo "ERROR: database '${DB_NAME}' doesn't exist!"
        else
            rm -f ${tmp_file}

            # output table content
            echo "The follwing entries have been added to '${DB_TABLE}':"
            ${sqlite_bin} ${DB_NAME} "SELECT DISTINCT ipaddr FROM ${DB_TABLE} WHERE ipaddr NOT IN (SELECT DISTINCT ipaddr FROM ${DB_TABLE_TMP});" | \
            sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4
            echo
            echo "The follwing entries have been deleted from '${DB_TABLE}':"
            ${sqlite_bin} ${DB_NAME} "SELECT DISTINCT ipaddr FROM ${DB_TABLE_TMP} WHERE ipaddr NOT IN (SELECT DISTINCT ipaddr FROM ${DB_TABLE});" | \
            sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4
            rm -f ${tmp_file}
        fi
        rm -f /$bfbhome/bfb-db.lck
        ;;

    print )
        # output database content
        if [ ! -f ${DB_NAME} ]
        then
            echo "ERROR: database '${DB_NAME}' doesn't exist!"
        else
            var=$2
            # output table content
            if [ "$var" = "quiet" ]
            then
                ${sqlite_bin} ${DB_NAME} "SELECT ipaddr FROM ${DB_TABLE};" | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4 >/dev/null
            elif [ "$var" = "atma" -o "$var" = "proaktiv" ]
            then
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE_Z};" | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4
            elif [ "$var" = "block" -o "$var" = "free" -o "$var" = "slow" ]
            then
                ${sqlite_bin} ${DB_NAME} "SELECT ipaddr FROM ${DB_TABLE} WHERE reason = '${var}';" | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4
            else
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE};" | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4
            fi
        fi
        rm -f /$bfbhome/bfb-db.lck
        ;;

    

    display )
        ## used by setup menue
        if [ "$3" = "" ]
        then
            IP=l
        else
            IP=$3
        fi
        if [ ! -f ${DB_NAME} ]
        then
            echo "ERROR: database '${DB_NAME}' doesn't exist!"
        else
            if [ "$2" = "anzahl" -o "$2" = "Anzahl" ]
            then
                echo "Insgesamt:"
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE_Z};" | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4 | awk -F"|" '{print $1" Anzahl: "$2" Last used: "$4" reason: "$3}' |grep -Ev "DD|free" |grep -F "$IP"
                echo "aktuell geblockt:"
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE};" | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4 | awk -F"|" '{print $1" Anzahl: "$2" Last used: "$4" reason: "$3}' |grep -Ev "DD|free" |grep -F "$IP"
            elif [ "$2" = "anzahllong" ]
            then
                echo "Insgesamt:"
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE_Z};" | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4 | awk -F"|" '{print $1" Anzahl: "$2" Last used: "$4" reason: "$3}' |grep -Ev "DD|free" |grep -F "$IP"
            else
                echo "Insgesamt:"
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE_Z};"|grep atma | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4 | awk -F"|" '{print $1}'
                echo "aktuell geblockt:"
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE};"|grep block | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4 | awk -F"|" '{print $1}'
                ${sqlite_bin} ${DB_NAME} "SELECT * FROM ${DB_TABLE};"|grep slow | sort -n -t . -k 1,1 -k 2,2 -k 3,3 -k 4,4 | awk -F"|" '{print $1}'
           fi
        fi
        rm -f /$bfbhome/bfb-db.lck
        ;;
    show )
            ip=$2
            var=$3
            if [ "$var" = "quiet" ]
            then
                ${sqlite_bin} ${DB_NAME} "SELECT ipaddr FROM ${DB_TABLE} WHERE ipaddr = '${ip}';" >/dev/null
            elif [ "$var" = "anzahl" -o "$var" = "Anzahl" ]
            then
                ANZAHLV=`${sqlite_bin} ${DB_NAME} "SELECT count FROM ${DB_TABLE_Z} WHERE ipaddr = '${ip}';"`
                #echo ANZAHLV=$ANZAHLV
                if [ "$ANZAHLV" = "" ]
                then
                    ANZAHLV2=0
                else
                    ANZAHLV2=$ANZAHLV
                fi
                echo "$ip: $ANZAHLV2"
            elif [ "$var" = "reason" ]
            then
                ${sqlite_bin} ${DB_NAME} "SELECT ipaddr FROM ${DB_TABLE} WHERE reason = '${ip}';"    
            elif [ "$var" = "reasonip" ]
            then
                if [[ `echo $ip |grep ^fe80` ]]
                then
                    echo $ip
                else
                    ${sqlite_bin} ${DB_NAME} "SELECT reason,ipaddr FROM ${DB_TABLE} WHERE reason = '${ip}';"
                fi
            else
                ${sqlite_bin} ${DB_NAME} "SELECT ipaddr FROM ${DB_TABLE} WHERE ipaddr = '${ip}' and NOT reason = 'install';"
            fi
            rm -f /$bfbhome/bfb-db.lck
        ;;
    * )
        echo "Syntax error: Use print|show|add|del|init [force]|display Anzahl[IP]"
        ;;
esac
rm -f /$bfbhome/bfb-db.lck
